@@ -22,8 +22,8 @@ struct Options{
 	
 	bool isPaired, useAdapterFile, useNumberTag, useRemovalTag, umiTags, logStdout;
 	bool switch2Fasta, writeUnassigned, writeSingleReads, writeSingleReadsP, writeLengthDist;
-	bool useStdin, useStdout, relaxRegion, useRcTrimEnd, qtrimPostRm;
-	bool interleavedInput, htrimAdapterRm, htrimMaxFirstOnly;
+	bool useStdin, useStdout, relaxRegion, useRcTrimEnd, qtrimPostRm, addBarcodeAdapter;
+	bool interleavedInput, iupacInput, htrimAdapterRm, htrimMaxFirstOnly;
 	
 	int cutLen_begin, cutLen_end, cutLen_read, a_tail_len, b_tail_len, p_min_overlap;
 	int qtrimThresh, qtrimWinSize, a_overhang, htrimMinLength, htrimMinLength2, htrimMaxLength;
@@ -44,6 +44,7 @@ struct Options{
 	flexbar::RevCompMode     rcMode;
 	flexbar::PairOverlap     poMode;
 	flexbar::AdapterPreset   aPreset;
+	flexbar::AdapterTrimmed  aTrimmed;
 	
 	tbb::concurrent_vector<flexbar::TBar> barcodes, adapters, barcodes2, adapters2;
 	
@@ -80,10 +81,12 @@ struct Options{
 		logStdout         = false;
 		umiTags           = false;
 		interleavedInput  = false;
+		iupacInput        = false;
 		useStdin          = false;
 		useStdout         = false;
 		relaxRegion       = false;
 		useRcTrimEnd      = false;
+		addBarcodeAdapter = false;
 		qtrimPostRm       = false;
 		htrimAdapterRm    = false;
 		htrimMaxFirstOnly = false;
@@ -114,6 +117,7 @@ struct Options{
 		arc_end   = RIGHT;
 		b_end     = LTAIL;
 		aPreset   = APOFF;
+		aTrimmed  = ATON;
     }
 };
 
@@ -149,7 +153,7 @@ const std::string getFlexbarURL(){
 
 
 const std::string getFlexbarDescription(){
-	return "The program Flexbar preprocesses high-throughput sequencing data efficiently. It demultiplexes barcoded runs and removes adapter sequences. Moreover, trimming and filtering features are provided. Flexbar increases read mapping rates and improves genome as well as transcriptome assemblies. Unique molecular identifiers can be extracted in a flexible way. The program supports sequencing data in fasta and fastq format, e.g. from the Illumina platform. Refer to the manual on github.com/seqan/flexbar/wiki or contact Johannes Roehr on github.com/jtroehr for support with this application.";
+	return "The program Flexbar preprocesses high-throughput sequencing data efficiently. It demultiplexes barcoded runs and removes adapter sequences. Several adapter removal presets for Illumina libraries are included. Flexbar computes exact overlap alignments using SIMD and multicore parallelism. Moreover, trimming and filtering features are provided, e.g. trimming of homopolymers at read ends. Flexbar increases read mapping rates and improves genome as well as transcriptome assemblies. Unique molecular identifiers can be extracted in a flexible way. The software supports data in fasta and fastq format from multiple sequencing platforms. Refer to the manual on github.com/seqan/flexbar/wiki or contact Johannes Roehr on github.com/jtroehr for support with this application.";
 }
 
 
@@ -184,6 +188,7 @@ void defineOptions(seqan::ArgumentParser &parser, const std::string version, con
 	addOption(parser, ArgParseOption("r", "reads", "Fasta/q file or stdin (-) with reads that may contain barcodes.", ARG::INPUT_FILE));
 	addOption(parser, ArgParseOption("p", "reads2", "Second input file of paired reads, gz and bz2 files supported.", ARG::INPUT_FILE));
 	addOption(parser, ArgParseOption("i", "interleaved", "Interleaved format for first input set with paired reads."));
+	addOption(parser, ArgParseOption("I", "iupac", "Accept iupac symbols in reads and convert to N if not ATCG."));
 	
 	addSection(parser, "Barcode detection");
 	addOption(parser, ArgParseOption("b",  "barcodes", "Fasta file with barcodes for demultiplexing, may contain N.", ARG::INPUT_FILE));
@@ -214,7 +219,9 @@ void defineOptions(seqan::ArgumentParser &parser, const std::string version, con
 	addOption(parser, ArgParseOption("av", "adapter-min-poverlap", "Minimum overlap of paired reads for detection.", ARG::INTEGER));
 	addOption(parser, ArgParseOption("ac", "adapter-revcomp", "Include reverse complements of adapters.", ARG::STRING));
 	addOption(parser, ArgParseOption("ad", "adapter-revcomp-end", "Use different trim-end for reverse complements of adapters.", ARG::STRING));
+	addOption(parser, ArgParseOption("ab", "adapter-add-barcode", "Add reverse complement of detected barcode to adapters."));
 	addOption(parser, ArgParseOption("ar", "adapter-read-set", "Consider only single read set for adapters.", ARG::STRING));
+	addOption(parser, ArgParseOption("ak", "adapter-trimmed-out", "Modify that trimmed reads are kept.", ARG::STRING));
 	addOption(parser, ArgParseOption("ay", "adapter-cycles", "Number of adapter removal cycles.", ARG::INTEGER));
 	addOption(parser, ArgParseOption("am", "adapter-match", "Alignment match score.", ARG::INTEGER));
 	addOption(parser, ArgParseOption("ai", "adapter-mismatch", "Alignment mismatch score.", ARG::INTEGER));
@@ -257,7 +264,7 @@ void defineOptions(seqan::ArgumentParser &parser, const std::string version, con
 	addOption(parser, ArgParseOption("z", "zip-output", "Direct compression of output files.", ARG::STRING));
 	addOption(parser, ArgParseOption("1", "stdout-reads", "Write reads to stdout, tagged and interleaved if needed."));
 	addOption(parser, ArgParseOption("R", "output-reads", "Output file for reads instead of target prefix usage.", ARG::OUTPUT_FILE));
-	addOption(parser, ArgParseOption("D", "output-reads2", "Output file for reads2 instead of target prefix usage.", ARG::OUTPUT_FILE));
+	addOption(parser, ArgParseOption("P", "output-reads2", "Output file for reads2 instead of target prefix usage.", ARG::OUTPUT_FILE));
 	addOption(parser, ArgParseOption("j", "length-dist", "Generate length distribution for read output files."));
 	addOption(parser, ArgParseOption("s", "single-reads", "Write single reads for too short counterparts in pairs."));
 	addOption(parser, ArgParseOption("S", "single-reads-paired", "Write paired single reads with N for short counterparts."));
@@ -287,6 +294,8 @@ void defineOptions(seqan::ArgumentParser &parser, const std::string version, con
 	setAdvanced(parser, "adapter-min-poverlap");
 	setAdvanced(parser, "adapter-revcomp");
 	setAdvanced(parser, "adapter-revcomp-end");
+	setAdvanced(parser, "adapter-add-barcode");
+	setAdvanced(parser, "adapter-trimmed-out");
 	setAdvanced(parser, "adapter-read-set");
 	setAdvanced(parser, "adapter-cycles");
 	setAdvanced(parser, "adapter-match");
@@ -308,6 +317,7 @@ void defineOptions(seqan::ArgumentParser &parser, const std::string version, con
 	setAdvanced(parser, "bundle");
 	setAdvanced(parser, "bundles");
 	setAdvanced(parser, "interleaved");
+	setAdvanced(parser, "iupac");
 	setAdvanced(parser, "length-dist");
 	setAdvanced(parser, "single-reads");
 	setAdvanced(parser, "single-reads-paired");
@@ -359,6 +369,7 @@ void defineOptions(seqan::ArgumentParser &parser, const std::string version, con
 	
 	setValidValues(parser, "adapter-read-set", "1 2");
 	setValidValues(parser, "adapter-revcomp", "ON ONLY");
+	setValidValues(parser, "adapter-trimmed-out", "OFF ONLY");
 	setValidValues(parser, "adapter-pair-overlap", "ON SHORT ONLY");
 	setValidValues(parser, "adapter-preset", "TruSeq SmallRNA Methyl Ribo Nextera NexteraMP");
 	
@@ -403,7 +414,7 @@ void defineOptions(seqan::ArgumentParser &parser, const std::string version, con
 	addText(parser._toolDoc, "\\fBflexbar\\fP \\fB-r\\fP reads.fq \\fB-t\\fP target \\fB-q\\fP TAIL \\fB-qf\\fP i1.8", false);
 	addText(parser._toolDoc, "\\fBflexbar\\fP \\fB-r\\fP reads.fq \\fB-b\\fP barcodes.fa \\fB-bt\\fP LTAIL", false);
 	addText(parser._toolDoc, "\\fBflexbar\\fP \\fB-r\\fP reads.fq \\fB-a\\fP adapters.fa \\fB-ao\\fP 3 \\fB-ae\\fP 0.1", false);
-	addText(parser._toolDoc, "\\fBflexbar\\fP \\fB-r\\fP r1.fq \\fB-p\\fP r2.fq \\fB-a\\fP adapters.fa \\fB-ap\\fP ON", false);
+	addText(parser._toolDoc, "\\fBflexbar\\fP \\fB-r\\fP r1.fq \\fB-p\\fP r2.fq \\fB-a\\fP a1.fa \\fB-a2\\fP a2.fa \\fB-ap\\fP ON", false);
 	addText(parser._toolDoc, "\\fBflexbar\\fP \\fB-r\\fP r1.fq \\fB-p\\fP r2.fq \\fB-aa\\fP TruSeq \\fB-ap\\fP ON");
 }
 
@@ -613,6 +624,11 @@ void loadOptions(Options &o, seqan::ArgumentParser &parser){
 		}
 	}
 	
+	if(isSet(parser, "iupac")){
+		*out << "Iupac reads:           on" << endl;
+		o.iupacInput = true;
+	}
+	
 	
 	// barcode and adapter file options
 	
@@ -871,12 +887,10 @@ void loadOptions(Options &o, seqan::ArgumentParser &parser){
 		
 		if(isSet(parser, "output-reads") && (isSet(parser, "output-reads2") || o.runType == SINGLE)){
 			getOptionValue(o.outReadsFile, parser, "output-reads");
-			*out << "output-reads:          " << o.outReadsFile << endl;
 		}
 		
 		if(isSet(parser, "output-reads2") && isSet(parser, "output-reads") && o.runType == PAIRED){
 			getOptionValue(o.outReadsFile2, parser, "output-reads2");
-			*out << "output-reads2:         " << o.outReadsFile2 << endl;
 			
 			if(o.outReadsFile == o.outReadsFile2){
 				cerr << "\n" << "Output reads and reads2 file should not be the same.\n" << endl;
@@ -1043,6 +1057,22 @@ void loadOptions(Options &o, seqan::ArgumentParser &parser){
 				o.relaxRegion = true;
 			}
 			
+			if(isSet(parser, "adapter-add-barcode") && o.isPaired && o.a_end == RIGHT && o.rcMode != RCON &&
+				o.barDetect != BARCODE_READ && o.barDetect != BOFF && o.b_end == LTAIL){
+				
+				*out << "adapter-add-barcode:   on" << endl;
+				o.addBarcodeAdapter = true;
+			}
+			
+			if(isSet(parser, "adapter-trimmed-out")){
+				string a_trimmed_out;
+				getOptionValue(a_trimmed_out, parser, "adapter-trimmed-out");
+				*out << "adapter-trimmed-out:   " << a_trimmed_out << endl;
+				
+				     if(a_trimmed_out == "OFF")  o.aTrimmed = ATOFF;
+				else if(a_trimmed_out == "ONLY") o.aTrimmed = ATONLY;
+			}
+			
 			if(isSet(parser, "adapter-read-set") && o.isPaired && o.adapRm != NORMAL2){
 				string a_read_set;
 				getOptionValue(a_read_set, parser, "adapter-read-set");
