@@ -16,7 +16,7 @@ private:
 	const flexbar::QualTrimType m_qtrim;
 	const flexbar::FileFormat m_format;
 	
-	const bool m_preProcess, m_useStdin, m_qtrimPostRm;
+	const bool m_preProcess, m_useStdin, m_qtrimPostRm, m_iupacInput;
 	const int m_maxUncalled, m_preTrimBegin, m_preTrimEnd, m_qtrimThresh, m_qtrimWinSize;
 	tbb::atomic<unsigned long> m_nrReads, m_nrChars, m_nLowPhred;
 	
@@ -33,6 +33,7 @@ public:
 		m_qtrimThresh(o.qtrimThresh),
 		m_qtrimWinSize(o.qtrimWinSize),
 		m_qtrimPostRm(o.qtrimPostRm),
+		m_iupacInput(o.iupacInput),
 		m_format(o.format),
 		m_nrReads(0),
 		m_nrChars(0),
@@ -76,12 +77,30 @@ public:
 				reserve(seqs,     nReads);
 				reserve(uncalled, nReads);
 				
-				if(m_format == FASTA){
-					readRecords(ids, seqs, seqFileIn, nReads);
+				if(! m_iupacInput){
+					
+					if(m_format == FASTA){
+						readRecords(ids, seqs, seqFileIn, nReads);
+					}
+					else{
+						reserve(quals, nReads);
+						readRecords(ids, seqs, quals, seqFileIn, nReads);
+					}
 				}
 				else{
-					reserve(quals, nReads);
-					readRecords(ids, seqs, quals, seqFileIn, nReads);
+					seqan::StringSet<seqan::IupacString> seqsIupac;
+					
+					reserve(seqsIupac, nReads);
+					
+					if(m_format == FASTA){
+						readRecords(ids, seqsIupac, seqFileIn, nReads);
+					}
+					else{
+						reserve(quals, nReads);
+						readRecords(ids, seqsIupac, quals, seqFileIn, nReads);
+					}
+					
+					seqs = seqsIupac;
 				}
 				
 				for(unsigned int i = 0; i < length(ids); ++i){
@@ -133,6 +152,7 @@ public:
 						}
 					}
 				}
+				
 				m_nrReads += length(ids);
 				
 				return length(ids);
